@@ -1,5 +1,5 @@
 ;;; beanshell.el
-;; $Revision: 1.51 $ 
+;; $Revision: 1.52 $ 
 
 ;; Author: Paul Kinnucan <paulk@mathworks.com>
 ;; Maintainer: Paul Kinnucan
@@ -105,12 +105,10 @@ buffer."
 		   (append
 		    (list
 		     (expand-file-name "bsh-commands" jde-java-directory)
-		     (unless (eq system-type 'darwin) ;; darwin = Macintosh OS X 10.2.1
-		       (let ((tools 
-			    (expand-file-name "lib/tools.jar" (jde-get-jdk-dir))))
-			 (if (file-exists-p tools)
-			     tools
-			   (error "Cannot find JDK's tools jar file. See jde-get-jdk-dir."))))
+                     (let ((tools (jde-get-tools-jar)))
+                       (if (file-exists-p tools)
+                         tools
+                         (error "Cannot find JDK's tools jar file (or equivalent). See jde-get-jdk-dir.")))
 		     (if ant-home (expand-file-name "lib/ant.jar" ant-home))
 		     (expand-file-name "lib/checkstyle-all.jar" jde-java-directory)
 		     (expand-file-name "lib/jakarta-regexp.jar" jde-java-directory)
@@ -390,6 +388,9 @@ t."
 (provide 'beanshell)
 
 ;; $Log: beanshell.el,v $
+;; Revision 1.52  2002/12/06 03:47:37  ahyatt
+;; Changes to support Mac OS X, which does not use tools.jar
+;;
 ;; Revision 1.51  2002/11/14 06:03:34  paulk
 ;; Fix regression bug in bsh-exit caused by deferring loading of ant package until used.
 ;;
