@@ -1,6 +1,6 @@
 ;; jde-ant.el --- Use Apache Ant to build your JDE projects
 
-;; $Revision: 1.47 $ $Date: 2002/12/01 13:52:24 $ 
+;; $Revision: 1.50 $ $Date: 2002/12/06 03:47:36 $ 
 
 ;;
 ;; Author: Jason Stell | jason.stell@globalone.net
@@ -233,6 +233,13 @@ describing how the compilation finished"
   :group 'jde-ant
   :type 'hook)
 
+(defcustom jde-ant-working-directory ""
+  "*Path of the working directory to use in 'ant' build mode. This string must
+end in a slash, for example, c:/foo/bar/ or ./ . If this string is empty, the
+'ant' build mode uses the current file location as its working directory."
+  :group 'jde-ant
+  :type 'string)
+
 (defun jde-build-ant-command (target more-args &optional buildfile)
   "Constructs the java ant command. The variable `jde-ant-home' is used
 if it is set, otherwise it gets the ant home from the environment
@@ -299,7 +306,7 @@ variable ANT_HOME."
                                   (expand-file-name "lib/crimson.jar" ant-home)
                                   (expand-file-name "lib/jaxp.jar" ant-home)
                                   (expand-file-name "lib/optional.jar" ant-home)
-                                  (expand-file-name "lib/tools.jar" (jde-get-jdk-dir)))
+                                  (jde-get-tools-jar))
                             jde-ant-user-jar-files))
 
     (when jde-ant-use-global-classpath
@@ -328,7 +335,8 @@ variable ANT_HOME."
                       (file-name-directory jde-ant-interactive-buildfile))
                 (setq prompt-filename
                       (file-name-nondirectory jde-ant-interactive-buildfile)))
-            (setq prompt-directory default-directory)
+
+            (setq prompt-directory (jde-ant-get-default-directory))
             (setq prompt-filename ""))
           
           (setq buildfile
@@ -337,7 +345,8 @@ variable ANT_HOME."
         
       (if jde-ant-enable-find 
           (progn 
-            (setq buildfile (jde-ant-find-build-file default-directory))
+            (setq buildfile (jde-ant-find-build-file
+                             (jde-ant-get-default-directory)))
 
             (when (null buildfile)
               (error "Could not find Ant build file"))
@@ -479,8 +488,8 @@ and there are no more errors. "
               (if (not jde-xemacsp) compilation-file-regexp-alist))
 	(setq nomessage-regexp-alist
               (if (not jde-xemacsp) compilation-nomessage-regexp-alist))
-    
-        (let* (proc (thisdir default-directory) outwin)
+  
+        (let* (proc (thisdir (jde-ant-get-default-directory)) outwin)
           (save-excursion
             ;; Clear out the compilation buffer and make it writable.
             (if (not (bsh-running-p))
@@ -611,7 +620,6 @@ function uses the same rules as `jde-ant-build' for finding the buildfile."
 (defun jde-ant-find-build-file (dir)
   "Find the next Ant build file upwards in the directory tree from DIR.
 Returns nil if it cannot find a project file in DIR or an ascendant directory."
-
   (let ((file (find (cond ((string= jde-ant-buildfile "") "build.xml")
                           (t jde-ant-buildfile))
                     (directory-files dir) :test 'string=)))
@@ -646,6 +654,13 @@ Returns nil if it cannot find a project file in DIR or an ascendant directory."
   (interactive)
   (customize-apropos "jde-ant" 'groups))
 
+(defun jde-ant-get-default-directory ()
+  "Gets the default-directory according to the value of
+'jde-ant-working-directory."
+  (if (string= jde-ant-working-directory "")
+      default-directory
+    jde-ant-working-directory))
+
 ;; Register and initialize the customization variables defined
 ;; by this package.
 (jde-update-autoloaded-symbols)
@@ -656,6 +671,20 @@ Returns nil if it cannot find a project file in DIR or an ascendant directory."
 
 ;;
 ;; $Log: jde-ant.el,v $
+;; Revision 1.50  2002/12/06 03:47:36  ahyatt
+;; Changes to support Mac OS X, which does not use tools.jar
+;;
+;; Revision 1.49  2002/12/05 19:34:15  jslopez
+;; Readds normalizing jde-ant-buildfile. The problem with normalizing occurs only
+;; when using the Emacs that comes with the cygwin distribution.  In that emacs a
+;; path that starts with a drive letter, i.e. C: is not properly interpreted by
+;; emacs.
+;;
+;; Revision 1.48  2002/12/02 14:26:22  jslopez
+;; Adds jde-ant-working-directory.
+;; Rollbacks previous change of normalizing the jde-ant-buildfile.
+;; In certain instances you endup with a wrong buildfile path.
+;;
 ;; Revision 1.47  2002/12/01 13:52:24  jslopez
 ;; Normalizes jde-ant-buildfile before using it.
 ;; Thanks to Luis Miguel Hernanz <luish@germinus.com> for the patch.
