@@ -1,5 +1,5 @@
 ;;; jde-bug.el -- JDEbug Interface
-;; $Revision: 1.86 $ $Date: 2002/10/30 13:01:41 $ 
+;; $Revision: 1.87 $ $Date: 2002/12/07 17:09:44 $ 
 
 ;; Author: Paul Kinnucan <paulk@mathworks.com>
 ;; Maintainer: Paul Kinnucan
@@ -695,6 +695,7 @@ You can use the notation [f1], [f2], etc., to specify function keys."
     (oset process startupp nil)
     (oset process suspendedp nil)
     (oset process steppablep nil)
+    (setq overlay-arrow-position nil)
     (jde-dbs-cmd-exec run)))
 
 
@@ -2588,6 +2589,10 @@ of the variable `jde-bug-stack-info'"
 (provide 'jde-bug)
 
 ;; $Log: jde-bug.el,v $
+;; Revision 1.87  2002/12/07 17:09:44  jslopez
+;; Fixes bug ID 4. The overlay arrow nows get cleared after using
+;; jde-bug-continue.
+;;
 ;; Revision 1.86  2002/10/30 13:01:41  paulk
 ;; Moved XEmacs compatibility fix for beginning/end-of-line functions
 ;; from this file to jde-db.el
