@@ -1,5 +1,5 @@
 ;; JDE-XREF.EL --- Class cross-reference commands for the JDEE.
-;; $Revision: 1.6 $ $Date: 2002/12/01 02:50:48 $
+;; $Revision: 1.9 $ $Date: 2002/12/06 03:24:45 $
 ;;
 ;; Copyright (C) 2002 Andrew Hyatt
 ;;
@@ -26,7 +26,7 @@
 ;; LCD Archive Entry:
 ;; jde-xref|Andrew Hyatt|
 ;; |Java class cross-referencing commands for the JDEE
-;; |$Date: 2002/12/01 02:50:48 $|$Revision: 1.6 $|~/packages/jde-xref.el
+;; |$Date: 2002/12/06 03:24:45 $|$Revision: 1.9 $|~/packages/jde-xref.el
 
 ;;; Commentary:
 
@@ -289,12 +289,15 @@ the strings in LIST exist in STR"
   (jde-xref-partial-match-member
    (replace-regexp-in-string "/\\|\\$" "." path) classes))
 
-(defun jde-xref-create-package-hashes ()
+(defun jde-xref-create-package-hashes (&optional fake)
   "Returns a list of the three hashes that are in a package's data.
-The hashes are for the caller-hash, the interface-hash and the member-hash"
-  (list (make-hash-table :test 'equal :size 100)
-        (make-hash-table :test 'equal :size 20)
-        (make-hash-table :test 'equal :size 100)))
+The hashes are for the caller-hash, the interface-hash and the
+member-hash.  FAKE determines if we are just creating them so that
+there is something to check against.  In those circumstance we just
+create tiny hashes to conserve memory."
+  (list (make-hash-table :test 'equal :size (if fake 1 100))
+        (make-hash-table :test 'equal :size (if fake 1 20))
+        (make-hash-table :test 'equal :size (if fake 1 100))))
 
 (defun jde-xref-load-package-hashes (package)
   (let ((data (jde-xref-create-package-hashes)))
@@ -364,8 +367,12 @@ The hashes are for the caller-hash, the interface-hash and the member-hash"
                                         (when (nth 2 called)
                                           (jde-parse-get-unqualified-name
                                            (nth 2 called)))
-                                        (mapcar 'jde-parse-get-unqualified-name
-                                                (nth 3 called))))
+                                        ;; We don't want to need to
+                                        ;; know the constructor args
+                                        ;; for anonymous classes
+                                        (unless (jde-xref-is-class-anonymous (car called))
+                                          (mapcar 'jde-parse-get-unqualified-name
+                                                  (nth 3 called)))))
                          (called-package (jde-parse-get-package-from-name
                                           (car dqcalled))))
                     ;; Create the package data if needed
@@ -430,9 +437,9 @@ interfaces to a function should not be considered"
                             (jde-xref-get-current-signature) strict))
   (jde-xref-next-caller))
 
-(defun jde-xref-goto-sig (sig)
-  (jde-find-class-source (car sig))
-  (goto-line (nth 4 sig)))
+(defun jde-xref-goto-caller (caller)
+  (jde-find-class-source (car caller))
+  (goto-line (nth 4 caller)))
 
 ;;;###autoload
 (defun jde-xref-next-caller ()
@@ -440,7 +447,7 @@ interfaces to a function should not be considered"
 and show it"
   (interactive)
   (if jde-xref-stack
-    (jde-xref-goto-sig (pop jde-xref-stack))
+    (jde-xref-goto-caller (pop jde-xref-stack))
     (message "No more calls")))
 
 (defun jde-xref-load-subclasses-table-if-necessary ()
@@ -483,15 +490,17 @@ and show it"
 (defun jde-xref-find-or-create-package-in-cache (package)
   (unless jde-xref-db-base-directory
     (error "The variable `jde-xref-db-base-directory' must be specified to load the xref db"))
-  (or (jde-xref-find-package-in-cache package jde-xref-cache)
-      ;; Or we need to get the new package and put it in the cache
-      (let ((data (jde-xref-load-package-hashes package)))
+  (if (jde-xref-substring-member package jde-xref-store-prefixes)
+    (or (jde-xref-find-package-in-cache package jde-xref-cache)
+        ;; Or we need to get the new package and put it in the cache
+        (let ((data (jde-xref-load-package-hashes package)))
         (setq jde-xref-cache (cons (cons package data)
                                    (if (> (length jde-xref-cache)
                                           jde-xref-cache-size)
-                                     (cdr jde-xref-cache)
+                                       (cdr jde-xref-cache)
                                      jde-xref-cache)))
-        data)))
+        data))
+    (jde-xref-create-package-hashes t)))
 
 (defun jde-xref-get-caller-hash (package)
   (nth 0 (jde-xref-find-or-create-package-in-cache package)))
@@ -510,59 +519,70 @@ and show it"
   (gethash class (jde-xref-get-member-hash (jde-parse-get-package-from-name
                                             class))))
 
+(defun jde-xref-is-class-anonymous (class)
+  (string-match "\\$[0-9]+$" class))
+
+(defun jde-xref-is-caller-anonymous-class (caller)
+  (jde-xref-is-class-anonymous (nth 0 caller)))
+
+(defun jde-xref-is-sig-anonymous-class (sig)
+  (jde-xref-is-class-anonymous (nth 1 sig)))
+
 (defun jde-xref-get-callers (sig &optional strict)
   (let ((package (jde-parse-get-package-from-name (nth 1 sig)))
         (interface-callers nil))
-    (append (unless strict
-              (dolist (interface (gethash (nth 1 sig)
-                                          (jde-xref-get-interface-hash
-                                           package))
-                       interface-callers)
-                (setq interface-callers
-                        (append interface-callers
-                                (jde-xref-get-basic-caller
-                                 (cons 'function
-                                       (cons interface (cddr sig))))))))
-            (jde-xref-get-basic-caller sig)
-            (mapcan (lambda (subclass)
-                      (unless (member (cddr sig)
-                                      (jde-xref-get-members
-                                       subclass))
-                        (jde-xref-get-callers
-                          `(,(car sig) ,subclass ,@(cddr sig)) strict)))
-                  (gethash (nth 1 sig) jde-xref-subclasses)))))
+    (append
+     ;; if we're an anonymous class, then we want to see where we are
+     ;; created, since it kind of goes along with usage of the function.
+     (when (jde-xref-is-sig-anonymous-class sig)
+       (jde-xref-get-basic-caller (list (nth 0 sig) (nth 1 sig)
+                                        "<init>" nil nil)))
+     (unless strict
+       (dolist (interface (gethash (nth 1 sig)
+                                   (jde-xref-get-interface-hash
+                                    package))
+                          interface-callers)
+         (setq interface-callers
+               (append interface-callers
+                       (jde-xref-get-basic-caller
+                        (cons 'function
+                              (cons interface (cddr sig))))))))
+     (jde-xref-get-basic-caller sig)
+     (mapcan (lambda (subclass)
+               (unless (member (cddr sig)
+                               (jde-xref-get-members
+                                subclass))
+                 (jde-xref-get-callers
+                  `(,(car sig) ,subclass ,@(cddr sig)) strict)))
+             (gethash (nth 1 sig) jde-xref-subclasses)))))
 
 (defun jde-xref-notify (widget child &optional event)
-  (jde-xref-goto-sig (widget-get widget :sig)))
+  (jde-xref-goto-caller (widget-get widget :caller)))
 
-(defun jde-xref-tree-get-children (sig &optional tree)
-  (mapcan
-   (lambda (caller)
-     (if tree
-       (widget-create-child-and-convert
-        tree 
-        :node `(push-button
-                :tag ,(jde-xref-signature-to-string caller)
-                :sig ,caller
-                :format "%[%t%]\n"
-                :notify jde-xref-notify)
-        :has-children (jde-xref-get-callers caller)
-        :sig caller
-        :dynargs 'jde-xref-tree-get-children-from-tree)
-       (list
-        'tree-widget
-        :node `(push-button
-                :tag ,(jde-xref-signature-to-string caller)
-                :sig ,caller
-                :format "%[%t%]\n"
-                :notify jde-xref-notify)
-        :has-children (jde-xref-get-callers caller)
-        :dynargs 'jde-xref-tree-get-children-from-tree
-        :sig caller)))
-   (jde-xref-get-callers sig)))
+(defun jde-xref-caller-to-sig (caller)
+  (list 'function (nth 0 caller) (nth 1 caller) (when (nth 2 caller) (jde-parse-get-unqualified-name (nth 2 caller)))
+        (mapcar 'jde-parse-get-unqualified-name (nth 3 caller))))
+
+(defun jde-xref-tree-get-children (sig)
+  (when sig
+    (mapcar
+     (lambda (caller)
+       (let  ((caller-sig (jde-xref-caller-to-sig caller)))
+         (list
+          'tree-widget
+          :node `(push-button
+                  :tag ,(jde-xref-signature-to-string caller-sig)
+                  :format "%[%t%]\n"
+                  :sig ,caller-sig
+                  :caller ,caller
+                  :notify jde-xref-notify)
+          :dynargs 'jde-xref-tree-get-children-from-tree
+          :sig caller-sig
+          :has-children t)))
+     (jde-xref-get-callers sig))))
   
 (defun jde-xref-tree-get-children-from-tree (tree)
-  (jde-xref-tree-get-children (widget-get tree :sig) tree))
+  (jde-xref-tree-get-children (widget-get tree :sig)))
 
 ;;;###autoload
 (defun jde-xref-display-call-tree (strict)
@@ -578,9 +598,8 @@ and show it"
     (widget-create 'tree-widget
                    :tag (jde-xref-signature-to-string sig)
                    :dynargs 'jde-xref-tree-get-children-from-tree
-                   :children (jde-xref-get-callers caller)
-                   :sig sig
-                   (jde-xref-tree-get-children sig))
+                   :has-children t
+                   :sig sig)
     (use-local-map widget-keymap)
     (widget-setup)))
 
@@ -694,6 +713,16 @@ call list of all files modified in emacs"
 (provide 'jde-xref)
 
 ;; $Log: jde-xref.el,v $
+;; Revision 1.9  2002/12/06 03:24:45  ahyatt
+;; Fixed bug in call-tree, where no expansion could take place after the
+;; 2nd level. Also renamed the argument to jde-xref-goto-caller, which takes
+;; a caller, not a sig.
+;;
+;; Revision 1.8  2002/12/04 04:29:19  ahyatt
+;; Got the call-tree to finally work.  Also made some adjustements for
+;; anonymous classes.  These adjustments probably will disappear with
+;; some later rework I have planned.
+;;
 ;; Revision 1.6  2002/12/01 02:50:48  ahyatt
 ;; Major revision - I realized that the previous version had neglected to
 ;; factor in subclasses of a class when computing the callers to a
